! $Id: alignment.prg,v 1.3 1999/08/31 08:29:31 cguirao Exp $
! $Name: FIASCO-1_3 $
!
! AUTHOR: C.Guirao
!
! NAME: alignment.prg 
!   Procedure to detect the central row and fwhm of an image at different
!   wavelength values (X axis): beginning, medium and end. 
!   The input image is assumed to be rotated in such a way that the dispersion
!   direction is oriented along the rows. The wavelength must be increasing 
!   from the left to the right and the STEP descriptors of the image must be
!   positive.
!   The procedure is useful to check the alignment and focus in rows of the
!   spectra (do not use it with calibration lamps)
!
! INPUT: 
!   Bi-dimensional FITS or BDF image with an long spectrum oriented in rows.
!   Threshold above background for detecting spectrum central line.
!
! OUTPUT: 
!   A table called gauss.table with XCEN indicating the central row for 
!   the expetrum and XFWHM with the fwhm.
!
!******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS or BDF image of a frame
! p2 OPTIONAL: Threshold for SET/LONG 
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ?	IMA "Enter image with expectrum: "
 DEFINE/PARA p2 1000	NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20         "alignment"
 DEFINE/LOCA fitsframe/c/1/40   {p1}
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA threshold/i/1/1    {p2}
 DEFINE/LOCA size_ccd/i/1/2
 DEFINE/LOCA width/i/1/1   

!
! Do not remove +lower_levels for 'y', SEARCH/LONG does not work without it
!
 DEFINE/LOCA y/r/1/1		0.0  ? +lower_levels
 DEFINE/LOCA start/i/1/1
 DEFINE/LOCA medium/i/1/1
 DEFINE/LOCA end/i/1/1
 DEFINE/LOCA column/i/1/1
 DEFINE/LOCA xstart/i/1/1
 DEFINE/LOCA xend/i/1/1	
 DEFINE/LOCA reply/c/1/1
 DEFINE/LOCA central_y/r/1/1
 DEFINE/LOCA fwhm_y/r/1/1
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
!
! Gets basename and extension of input file
!
 @@ file_ext {fitsframe}
 frame = basename
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   @@ compute_fits {fitsframe} {frame}
 ENDIF
!
! Get the size in pixels of the CCD, and set start/medium/end for 'x' avoiding 
! borders
!
 size_ccd(1) = m$value({frame},npix(1))
 size_ccd(2) = m$value({frame},npix(2))
 WRITE/OUT "{cmd}: {frame} CCD Size X:{size_ccd(1)} Y:{size_ccd(2)}"
!
! Issue a warning if Y-START is not 1
!
 IF m$value({frame},start(2)) .ne. 1.0 THEN
   WRITE/OUT "{cmd}: WARNING: START(2) Y coordinates do not start at 1"
   WRITE/OUT "{cmd}: Forcing START(2) to 1"
   WRITE/DESCR {frame} start 1,1
 ENDIF
!
! Calculate start,medium and end of the X axis
!
 start =  {size_ccd(1)} / 10		! Discard the first 10%
 medium = {size_ccd(1)} / 2		
 end   = {size_ccd(1)} - (size_ccd(1) / 10)  ! Discard the last 10%
!
! Initiate contex LONG
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
!
! Repeat the process to calculate central row and fwhm for columns
! start/medium and end.
!
column = start
repeat:
!
! First we average all the columns except borders. Second,
! search for the maximum 'y' that indicates the row position of the spectrum
! finally extract the row 'y' that contains the spectrum.
!
 AVERAG/COLU out  = {frame} @{column},@{column} >NULL
 SEARCH/LONG out thres={threshold} >NULL
!
! SEARCH/LONG generates a table called line.tbl with as many rows as
! lines are detected. Being an image with only one spectrum, we should
! detect only one line, otherwise we may need to increase the THRESHOLD
! SHOW/TABLE reads the table generated by SEARCH/LONG, the value keyword
! OUTPUTI(2) indicates the number of rows thus the number of lines detected.
! If we detect more than 1 line, the threshold might be too low 
!
 SHOW/TABLE line.tbl >NULL
 IF OUTPUTI(2) .ne. 1 then
   WRITE/KEYW reply/c/1/1 y
   WRITE/OUT
   WRITE/OUT "WARNING: More than one line detected by SEARCH/LONG"
   INQUIRE/KEYW reply "Doubling current threshold of {threshold} [yn] (y)? "
   IF reply(:1) .eq. "y" THEN
     threshold = 2 * {threshold}
     GOTO repeat
   ELSEIF reply(:1) .eq. "n" THEN
     ECHO/OFF
     RETURN/EXIT
   ENDIF
 ENDIF
!
 COPY/TK line :x  @1 y/r/1/1 
!
! 'y' from COPY/TK contains the central wavelength. Let's create a table
! where we set :XSTART and :XEND to central wavelength +- 150 pixels. The
! command CENTER/GAUSS will tell us then the FWHM
! 
! xstart = {y} - 150.0
! xend = {y} + 150.0
 xstart = size_ccd(2)/10
 xend = size_ccd(2) - (size_ccd(2)/10)
!
 CREATE/TABL gauss
 CREATE/COLU gauss :XSTART
 CREATE/COLU gauss :XEND
 WRITE/TABL gauss :XSTART @1 {xstart}
 WRITE/TABL gauss :XEND   @1 {xend}
 CENTER/GAUS out,gauss gauss >NULL
 central_y = m$value(gauss,:XCEN,@1)
 fwhm_y = m$value(gauss,:XFWHM,@1)
!
 WRITE/OUT "{cmd}: {frame} at X={column} with center row Y={central_y} with FWHM={fwhm_y}"
 IF {column} .eq. {start}  THEN
   column = medium
   GOTO repeat
 ELSEIF {column} .eq. {medium} THEN
   column = end
   GOTO repeat
 ENDIF 
!
! Clean all temporary files
!
 IF ext .ne. ".bdf" THEN
   DELETE/IMA {frame} NO
 ENDIF
 $rm -f NULL
 $rm -f midlis*.prg
 $rm -f gauss.tbl
 $rm -f lntab.tbl
 $rm -f row.bdf
 $rm -f line.tbl
 $rm -f out.bdf
!
ECHO/OFF
