! $Id: wave_calib.prg,v 1.7 2002/01/30 09:11:50 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: C.Guirao & J.M.Alcala
!
! NAME: wave_calib.prg
!   Automatic wavelength calibration procedure of an image in FITS format 
!   containing the spectrum of a calibration lamp.
!   The input image is assumed to be rotated in such a way that the dispersion
!   direction is oriented along the rows. The wavelength must be increasing
!   from the left to the right and the STEP descriptors of the image must be
!   positive.
!
! INPUT:
!   - 1-2-dimensional FITS image with the spectrum of a calibration lamp 
!     oriented in rows.
!   - MIDAS table of the calibration lamp. Default "neon.tbl"
!   - Wave range. Default 6155,6670  
!   - Threshold for SET/LONG. Default 1000
!   - Width search window. Default 4
!   - Tolerance. Default 0.5
!
! OUTPUT:
!   A table 'session.tbl' resuming the calibration.
!
! USAGE: @@ wave_calib <lamp> [table] [wave_range] [threshold] [width] [tol]
! 
! EXAMPLES:
!   Fiasco with Fiber:200um Grating:600 c/mm Objective: 100mm
!      @@ wave_calib ne01.fit ? 5092,7338 1000 5
!   Fiasco with Fiber:200um Grating:1200 c/mm Objective: 200mm (defaults)
!      @@ wave_calib ne01.fit
!
!*******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS or BDF (1,2)dimension of image of spectrum of calibration lamp
! p2 OPTIONAL: Table containing lines of calib. lamp.
! p3 OPTIONAL: Wavelength range (in angstroms) low,high
! p4 OPTIONAL: Threshold. Value above noise to be identified as a line 
!              (SET/LONG THRES)
! p5 OPTIONAL: Width in pixels taken for each emission line (SET/LONG WIDTH) 
! p6 OPTIONAL: Tolerance for rejection of outliers (SET/LONG TOL)
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ? I "Enter (1,2)dimensional image with the spectrum of a calibration lamp? "
 DEFINE/PARA p2 neon.tbl  TAB 
 DEFINE/PARA p3 5144,7400 NUM
 DEFINE/PARA p4 500      NUM
 DEFINE/PARA p5 5         NUM
 DEFINE/PARA p6 0.5       NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20  	"wave_calib"
 DEFINE/LOCA fitsframe/c/1/40 	{p1}
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA calib_table/c/1/8	{p2}
 DEFINE/LOCA wave_range/r/1/2 	{p3}
 DEFINE/LOCA threshold/i/1/1 	{p4}
 DEFINE/LOCA width/i/1/1 	{p5}
 DEFINE/LOCA tolerance/r/1/1    {p6}
 DEFINE/LOCA y/r/1/1   		0.0  ? +lower_levels
 DEFINE/LOCA central_y/r/1/1   	0.0
 DEFINE/LOCA fwhm_y/r/1/1   	0.0
 DEFINE/LOCA naxis/i/1/1 	0,0
 DEFINE/LOCA size_ccd/i/1/2 	0,0
 DEFINE/LOCA indx/i/1/1 	0
 DEFINE/LOCA reply/c/1/1 	y
 DEFINE/LOCA avdisp/r/1/1	0
 DEFINE/LOCA wave_central/i/1/1	0
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10    	"" ? +lower_levels
!
! Gets basename and extension of input file
!
 @@ file_ext {fitsframe} 
 frame = basename
! 
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   @@ compute_fits {fitsframe} {frame}
 ENDIF
!
! Get the size in pixels of the CCD
!
 size_ccd(1) = m$value({frame},npix(1))
 size_ccd(2) = m$value({frame},npix(2))
 naxis = m$value({frame},naxis)
!
! Calculate the average dispersion and the central wavelength
!
 avdisp       =	({wave_range(2)} - {wave_range(1)}) / {size_ccd(1)}
 wave_central =	({wave_range(1)} + ({wave_range(2)} - {wave_range(1)}) / 2)
!
 WRITE/OUT 
 WRITE/OUT "FRAME       = {frame}"
 WRITE/OUT "CALIB TABLE = {calib_table}"
 WRITE/OUT "WAVE_RANGE  = {p3}"
 WRITE/OUT "WAVE_CENTRAL= {wave_central}"
 WRITE/OUT "NAXIS       = {naxis}"
 WRITE/OUT "CCD_SIZE    = {size_ccd(1)},{size_ccd(2)}"
 WRITE/OUT "THRESHOLD   = {threshold}"
 WRITE/OUT "WIDTH       = {width}"
 WRITE/OUT "AVDISP      = {avdisp}"
 WRITE/OUT "TOLERANCE   = {tolerance}"
 WRITE/OUT
!
 IF m$exist(calib_table) .eq. 0 THEN
   WRITE/OUT "{cmd}: ERROR: Calibration table {calib_table} not found"
   WRITE/OUT "Usage: @@ {cmd} frame calib_table waverange TRESH WIDTH TOL"
   ECHO/OFF
   RETURN/EXIT
 ENDIF
!
 askContinue:
 WRITE/KEYW reply/c/1/1 y
 WRITE/OUT "Creating a wavelength calibration session with above values"
 INQUIRE/KEYW reply "Continue [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO continue
 ELSEIF reply(:1) .eq. "n" THEN
   WRITE/OUT "Usage: @@ {cmd} frame calib_table waverange TRESH WIDTH TOL"
   ECHO/OFF
   RETURN/EXIT
 ELSE
   GOTO askContinue
 ENDIF
 continue: 

!
! If this is a 1-dimensional image (output of extract/rtra ? ? plot) 
! do not search for the central row.
!
 IF naxis .eq. 2 THEN
   !
   ! Search for row central containing the spectrum
   !
   @@ search {frame} 
   !
   central_y = m$value(gauss,:XCEN,@1)
   fwhm_y = m$value(gauss,:XFWHM,@1)
   !
   EXTRAC/IMAG row = {frame} [@1,@{central_y}:@{size_ccd(1)},@{central_y}]
 ELSE
   compute/imag row = {frame}
 ENDIF
!
! Initiate contex LONG
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
! 
! Create a graphical window.
!
 GRAPH/SPEC
 SET/GRAPH pm=1 fount=1 xa=auto ya=auto
!
! Next is the setup for the LONG package to identify automatically the 
! wavelength range. Important values are:
!   Table with lines to be identified (e.g for He-Arg: hear.tbl)
!   Spectral range (e.g. from 4540 till 6780 angstroms)
!   1dim spectrum with lines to be identified (default: row.bdf)
!   Threshold: level that distinguish noise from signal 
!   Width of search window: A little bit above spectra resolution (default:4)
!   Search method (here Gauss)
!   See: SHOW/LONG
!
 SET/LONG LINCAT   = {calib_table}
 SET/LONG WRANG    = {wave_range(1)}
 SET/LONG WRANG    = ,{wave_range(2)}
 SET/LONG INSTRUME = FIASCO
 SET/LONG WLC      = row.bdf
 SET/LONG THRES    = {threshold}
 SET/LONG WIDTH    = {width}
 SET/LONG YWIDTH   = 0
 SET/LONG YSTEP    = 1
 SET/LONG SEAMTD   = GAUSS
 SET/LONG TOL      = {tolerance}
!
!
! Start searching lines that can be identified in the spectrum range
!
 SEARCH/LONG 
 PLOT/SEARCH
!
! Next is the command to identify automatically. Important is the values
! of 'avdisp'+/- 15% (average disparsion) and 'wcent'+/- 50 pixels 
! (wavelength center) which have to be accurate for a succesful identification
!
 SET/LONG wlcmtd=linear avdisp={avdisp} wcent={wave_central}
!
! The command ESTIMA/DISP returns a value for 'Confidence Level' the higher
! the better for a good identification
!
 ESTIMA/DISP 
 CALIBR/LONG 
 PLOT/RESI 
 PLOT/DELTA 
 PLOT/CALIB 
!
! Saving the results on session  
!
 SAVE/LONG session
!
! Uncomment next two lines if you want to calibrate the original 1d spectrum.
!
! REBIN/LONG row row_reb
! PLOT row_reb
!
!
 WRITE/OUT 
 WRITE/OUT "If temporary files are not removed you can re-execute commands like"
 WRITE/OUT "PLOT/DELT to see dispersion relation, "
 WRITE/OUT "PLOT/RESI to see residuals or"
 WRITE/OUT "PLOT/CALIB to see identified lines."
 WRITE/OUT 
!
 WRITE/OUT 
 WRITE/OUT "Wavelength calibration finished. Results in session.tbl"
!
 askRemove:
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "Remove all temporary files [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO remove
 ELSEIF reply(:1) .eq. "n" THEN
   ECHO/OFF
   RETURN/EXIT
 ELSE
   GOTO askRemove
 ENDIF
 remove: 
!
! Clean all temporary files 
!
 $rm -f NULL
 $rm -f gauss.tbl
 $rm -f row_reb.bdf
 $rm -f row.bdf 
 $rm -f line.tbl lntab.tbl 
 $rm -f graph_wnd001.plt long.plt
 $rm -f middumm*
!
 ECHO/OFF
